using System;

namespace CallByReference
{
	// This shows the difference between 
	//  - Call by Reference (you get the reference)
	//  - Pass Reference by Value (you get a copy of the pointer)
	//
	// The plus methods do the same both times:
	//  - add the Array Elements 1 and 2
	//  - increment Array Element 1 by 1
	//  - set the content of the passed Array to {23, 42}
	//
	// The outcome:
	/*
	 * Part I: CallByReference :::::::::::::::::::::
	 * before: a[0]: 7 a[1]: 8
	 * after:  a[0]: 23 a[1]: 42
	 *
	 * Part II: PassReferenceByValue :::::::::::::::
	 * before: a[0]: 7 a[1]: 8
	 * after:  a[0]: 8 a[1]: 8
	 * 
	 */
	class CallByReference
	{
		static int[] a;

		static void changeCallByReferenceAndNewArray(ref int[] x)
		{
			x[0]++;
			x = new int[]{23, 42};
		}

		static void changePassReferenceByValueAndNewArray(int[] x)
		{
			x[0]++;
			x = new int[]{23, 42};
		}

		[STAThread]
		static void Main(string[] args)
		{
			Console.WriteLine("Part I: CallByReference :::::::::::::::::::::");
			a = new int[]{7, 8};
			Console.WriteLine("before: a[0]: "+a[0]+" a[1]: "+a[1]);
			changeCallByReferenceAndNewArray(ref a);
			Console.WriteLine("after:  a[0]: "+a[0]+" a[1]: "+a[1]+"\n");
			
			Console.WriteLine("Part II: PassReferenceByValue :::::::::::::::");
			a = new int[]{7, 8};
			Console.WriteLine("before: a[0]: "+a[0]+" a[1]: "+a[1]);
			changePassReferenceByValueAndNewArray(a);
			Console.WriteLine("after:  a[0]: "+a[0]+" a[1]: "+a[1]);

			Console.ReadLine();
		}
	}
}
